@@ -24,7 +24,7 @@
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#include <ostream>
+#include <iostream>
 #include <bitset>
 
 #include <tstring.h>
@@ -35,140 +35,136 @@
 using namespace TagLib;
 using namespace APE;
 
-class Footer::FooterPrivate
+class APE::Footer::FooterPrivate
 {
 public:
-  FooterPrivate() : version(0),
-                    footerPresent(true),
-                    headerPresent(false),
-                    isHeader(false),
-                    itemCount(0),
-                    tagSize(0) {}
+  FooterPrivate() :
+    version(0),
+    footerPresent(true),
+    headerPresent(false),
+    isHeader(false),
+    itemCount(0),
+    tagSize(0) {}
 
-  ~FooterPrivate() {}
-
-  uint version;
+  unsigned int version;
 
   bool footerPresent;
   bool headerPresent;
 
   bool isHeader;
 
-  uint itemCount;
-  uint tagSize;
-
-  static const uint size = 32;
+  unsigned int itemCount;
+  unsigned int tagSize;
 };
 
 ////////////////////////////////////////////////////////////////////////////////
 // static members
 ////////////////////////////////////////////////////////////////////////////////
 
-TagLib::uint Footer::size()
+unsigned int APE::Footer::size()
 {
-  return FooterPrivate::size;
+  return 32;
 }
 
-ByteVector Footer::fileIdentifier()
+ByteVector APE::Footer::fileIdentifier()
 {
-  return ByteVector::fromCString("APETAGEX");
+  return ByteVector("APETAGEX");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // public members
 ////////////////////////////////////////////////////////////////////////////////
 
-Footer::Footer()
+APE::Footer::Footer() :
+  d(new FooterPrivate())
 {
-  d = new FooterPrivate;
 }
 
-Footer::Footer(const ByteVector &data)
+APE::Footer::Footer(const ByteVector &data) :
+  d(new FooterPrivate())
 {
-  d = new FooterPrivate;
   parse(data);
 }
 
-Footer::~Footer()
+APE::Footer::~Footer()
 {
   delete d;
 }
 
-TagLib::uint Footer::version() const
+unsigned int APE::Footer::version() const
 {
   return d->version;
 }
 
-bool Footer::headerPresent() const
+bool APE::Footer::headerPresent() const
 {
   return d->headerPresent;
 }
 
-bool Footer::footerPresent() const
+bool APE::Footer::footerPresent() const
 {
   return d->footerPresent;
 }
 
-bool Footer::isHeader() const
+bool APE::Footer::isHeader() const
 {
   return d->isHeader;
 }
 
-void Footer::setHeaderPresent(bool b) const
+void APE::Footer::setHeaderPresent(bool b) const
 {
   d->headerPresent = b;
 }
 
-TagLib::uint Footer::itemCount() const
+unsigned int APE::Footer::itemCount() const
 {
   return d->itemCount;
 }
 
-void Footer::setItemCount(uint s)
+void APE::Footer::setItemCount(unsigned int s)
 {
   d->itemCount = s;
 }
 
-TagLib::uint Footer::tagSize() const
+unsigned int APE::Footer::tagSize() const
 {
   return d->tagSize;
 }
 
-TagLib::uint Footer::completeTagSize() const
+unsigned int APE::Footer::completeTagSize() const
 {
   if(d->headerPresent)
-    return d->tagSize + d->size;
-  else
-    return d->tagSize;
+    return d->tagSize + size();
+  return d->tagSize;
 }
 
-void Footer::setTagSize(uint s)
+void APE::Footer::setTagSize(unsigned int s)
 {
   d->tagSize = s;
 }
 
-void Footer::setData(const ByteVector &data)
+void APE::Footer::setData(const ByteVector &data)
 {
   parse(data);
 }
 
-ByteVector Footer::renderFooter() const
+ByteVector APE::Footer::renderFooter() const
 {
-    return render(false);
+  return render(false);
 }
 
-ByteVector Footer::renderHeader() const
+ByteVector APE::Footer::renderHeader() const
 {
-    if (!d->headerPresent) return ByteVector();
-
-    return render(true);
+  if(!d->headerPresent)
+    return ByteVector();
+  return render(true);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // protected members
 ////////////////////////////////////////////////////////////////////////////////
 
-void Footer::parse(const ByteVector &data)
+void APE::Footer::parse(const ByteVector &data)
 {
   if(data.size() < size())
     return;
@@ -177,19 +173,19 @@ void Footer::parse(const ByteVector &data)
 
   // Read the version number
 
-  d->version = data.mid(8, 4).toUInt(false);
+  d->version = data.toUInt(8, false);
 
   // Read the tag size
 
-  d->tagSize = data.mid(12, 4).toUInt(false);
+  d->tagSize = data.toUInt(12, false);
 
   // Read the item count
 
-  d->itemCount = data.mid(16, 4).toUInt(false);
+  d->itemCount = data.toUInt(16, false);
 
   // Read the flags
 
-  std::bitset<32> flags(TAGLIB_CONSTRUCT_BITSET(data.mid(20, 4).toUInt(false)));
+  std::bitset<32> flags(TAGLIB_CONSTRUCT_BITSET(data.toUInt(20, false)));
 
   d->headerPresent = flags[31];
   d->footerPresent = !flags[30];
@@ -197,7 +193,7 @@ void Footer::parse(const ByteVector &data)
 
 }
 
-ByteVector Footer::render(bool isHeader) const
+ByteVector APE::Footer::render(bool isHeader) const
 {
   ByteVector v;
 
