@@ -5,7 +5,7 @@
 
 /***************************************************************************
  *   This library is free software; you can redistribute it and/or modify  *
- *   it  under the terms of the GNU Lesser General Public License version  *
+ *   it under the terms of the GNU Lesser General Public License version   *
  *   2.1 as published by the Free Software Foundation.                     *
  *                                                                         *
  *   This library is distributed in the hope that it will be useful, but   *
@@ -15,8 +15,12 @@
  *                                                                         *
  *   You should have received a copy of the GNU Lesser General Public      *
  *   License along with this library; if not, write to the Free Software   *
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
- *   USA                                                                   *
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
+ *   02110-1301  USA                                                       *
+ *                                                                         *
+ *   Alternatively, this file is available under the Mozilla Public        *
+ *   License Version 1.1.  You may obtain a copy of the License at         *
+ *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
 #include <tbytevectorlist.h>
@@ -30,7 +34,9 @@ using namespace APE;
 class APE::Item::ItemPrivate
 {
 public:
-  ItemPrivate() : type(Text), readOnly(false) {}
+  ItemPrivate() :
+    type(Text),
+    readOnly(false) {}
 
   Item::ItemTypes type;
   String key;
@@ -39,28 +45,45 @@ public:
   bool readOnly;
 };
 
-APE::Item::Item()
+////////////////////////////////////////////////////////////////////////////////
+// public members
+////////////////////////////////////////////////////////////////////////////////
+
+APE::Item::Item() :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
 }
 
-APE::Item::Item(const String &key, const String &value)
+APE::Item::Item(const String &key, const String &value) :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
   d->key = key;
   d->text.append(value);
 }
 
-APE::Item::Item(const String &key, const StringList &values)
+APE::Item::Item(const String &key, const StringList &values) :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
   d->key = key;
   d->text = values;
 }
 
-APE::Item::Item(const Item &item)
+APE::Item::Item(const String &key, const ByteVector &value, bool binary) :
+  d(new ItemPrivate())
+{
+  d->key = key;
+  if(binary) {
+    d->type = Binary;
+    d->value = value;
+  }
+  else {
+    d->text.append(value);
+  }
+}
+
+APE::Item::Item(const Item &item) :
+  d(new ItemPrivate(*item.d))
 {
-  d = new ItemPrivate(*item.d);
 }
 
 APE::Item::~Item()
@@ -70,11 +93,17 @@ APE::Item::~Item()
 
 Item &APE::Item::operator=(const Item &item)
 {
-  delete d;
-  d = new ItemPrivate(*item.d);
+  Item(item).swap(*this);
   return *this;
 }
 
+void APE::Item::swap(Item &item)
+{
+  using std::swap;
+
+  swap(d, item.d);
+}
+
 void APE::Item::setReadOnly(bool readOnly)
 {
   d->readOnly = readOnly;
@@ -100,14 +129,80 @@ String APE::Item::key() const
   return d->key;
 }
 
-ByteVector APE::Item::value() const
+ByteVector APE::Item::binaryData() const
 {
   return d->value;
 }
 
+void APE::Item::setBinaryData(const ByteVector &value)
+{
+  d->type = Binary;
+  d->value = value;
+  d->text.clear();
+}
+
+ByteVector APE::Item::value() const
+{
+  // This seems incorrect as it won't be actually rendering the value to keep it
+  // up to date.
+
+  return d->value;
+}
+
+void APE::Item::setKey(const String &key)
+{
+  d->key = key;
+}
+
+void APE::Item::setValue(const String &value)
+{
+  d->type = Text;
+  d->text = value;
+  d->value.clear();
+}
+
+void APE::Item::setValues(const StringList &value)
+{
+  d->type = Text;
+  d->text = value;
+  d->value.clear();
+}
+
+void APE::Item::appendValue(const String &value)
+{
+  d->type = Text;
+  d->text.append(value);
+  d->value.clear();
+}
+
+void APE::Item::appendValues(const StringList &values)
+{
+  d->type = Text;
+  d->text.append(values);
+  d->value.clear();
+}
+
 int APE::Item::size() const
 {
-  return 8 + d->key.size() + 1 + d->value.size();
+  int result = 8 + d->key.size() + 1;
+  switch(d->type) {
+    case Text:
+      if(!d->text.isEmpty()) {
+        StringList::ConstIterator it = d->text.begin();
+
+        result += it->data(String::UTF8).size();
+        it++;
+        for(; it != d->text.end(); ++it)
+          result += 1 + it->data(String::UTF8).size();
+      }
+      break;
+
+    case Binary:
+    case Locator:
+      result += d->value.size();
+      break;
+  }
+  return result;
 }
 
 StringList APE::Item::toStringList() const
@@ -115,22 +210,27 @@ StringList APE::Item::toStringList() const
   return d->text;
 }
 
+StringList APE::Item::values() const
+{
+  return d->text;
+}
+
 String APE::Item::toString() const
 {
-  return isEmpty() ? String::null : d->text.front();
+  if(d->type == Text && !isEmpty())
+    return d->text.front();
+  return String();
 }
 
 bool APE::Item::isEmpty() const
 {
   switch(d->type) {
-    case 0:
-    case 1:
+    case Text:
       if(d->text.isEmpty())
         return true;
-      if(d->text.size() == 1 && d->text.front().isEmpty())
-        return true;
-      return false;
-    case 2:
+      return d->text.size() == 1 && d->text.front().isEmpty();
+    case Binary:
+    case Locator:
       return d->value.isEmpty();
     default:
       return false;
@@ -146,31 +246,37 @@ void APE::Item::parse(const ByteVector &data)
     return;
   }
 
-  uint valueLength  = data.mid(0, 4).toUInt(false);
-  uint flags        = data.mid(4, 4).toUInt(false);
+  const unsigned int valueLength  = data.toUInt(0, false);
+  const unsigned int flags        = data.toUInt(4, false);
 
-  d->key = String(data.mid(8), String::UTF8);
+  // An item key can contain ASCII characters from 0x20 up to 0x7E, not UTF-8.
+  // We assume that the validity of the given key has been checked.
 
-  d->value = data.mid(8 + d->key.size() + 1, valueLength);
+  d->key = String(&data[8], String::Latin1);
+
+  const ByteVector value = data.mid(8 + d->key.size() + 1, valueLength);
 
   setReadOnly(flags & 1);
   setType(ItemTypes((flags >> 1) & 3));
 
-  if(int(d->type) < 2)
-    d->text = StringList(ByteVectorList::split(d->value, '\0'), String::UTF8);
+  if(Text == d->type)
+    d->text = StringList(ByteVectorList::split(value, '\0'), String::UTF8);
+  else
+    d->value = value;
 }
 
 ByteVector APE::Item::render() const
 {
   ByteVector data;
-  TagLib::uint flags = ((d->readOnly) ? 1 : 0) | (d->type << 1);
+  unsigned int flags = ((d->readOnly) ? 1 : 0) | (d->type << 1);
   ByteVector value;
 
   if(isEmpty())
     return data;
 
-  if(d->type != Item::Binary) {
+  if(d->type == Text) {
     StringList::ConstIterator it = d->text.begin();
+
     value.append(it->data(String::UTF8));
     it++;
     for(; it != d->text.end(); ++it) {
@@ -184,7 +290,7 @@ ByteVector APE::Item::render() const
 
   data.append(ByteVector::fromUInt(value.size(), false));
   data.append(ByteVector::fromUInt(flags, false));
-  data.append(d->key.data(String::UTF8));
+  data.append(d->key.data(String::Latin1));
   data.append(ByteVector('\0'));
   data.append(value);
 
