@@ -32,10 +32,20 @@
 using namespace TagLib;
 using namespace ID3v1;
 
+namespace
+{
+  const ID3v1::StringHandler defaultStringHandler;
+  const ID3v1::StringHandler *stringHandler = &defaultStringHandler;
+}
+
 class ID3v1::Tag::TagPrivate
 {
 public:
-  TagPrivate() : file(0), tagOffset(-1), track(0), genre(255) {}
+  TagPrivate() :
+    file(0),
+    tagOffset(0),
+    track(0),
+    genre(255) {}
 
   File *file;
   long tagOffset;
@@ -45,18 +55,18 @@ public:
   String album;
   String year;
   String comment;
-  uchar track;
-  uchar genre;
-
-  static const StringHandler *stringHandler;
+  unsigned char track;
+  unsigned char genre;
 };
 
-const ID3v1::StringHandler *ID3v1::Tag::TagPrivate::stringHandler = new StringHandler;
-
 ////////////////////////////////////////////////////////////////////////////////
 // StringHandler implementation
 ////////////////////////////////////////////////////////////////////////////////
 
+StringHandler::StringHandler()
+{
+}
+
 String ID3v1::StringHandler::parse(const ByteVector &data) const
 {
   return String(data, String::Latin1).stripWhiteSpace();
@@ -64,26 +74,23 @@ String ID3v1::StringHandler::parse(const ByteVector &data) const
 
 ByteVector ID3v1::StringHandler::render(const String &s) const
 {
-  if(!s.isLatin1())
-  {
-    return ByteVector();
-  }
-
-  return s.data(String::Latin1);
+  if(s.isLatin1())
+    return s.data(String::Latin1);
+  return ByteVector();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // public methods
 ////////////////////////////////////////////////////////////////////////////////
 
-ID3v1::Tag::Tag() : TagLib::Tag()
+ID3v1::Tag::Tag() :
+  d(new TagPrivate())
 {
-  d = new TagPrivate;
 }
 
-ID3v1::Tag::Tag(File *file, long tagOffset) : TagLib::Tag()
+ID3v1::Tag::Tag(File *file, long tagOffset) :
+  d(new TagPrivate())
 {
-  d = new TagPrivate;
   d->file = file;
   d->tagOffset = tagOffset;
 
@@ -100,11 +107,11 @@ ByteVector ID3v1::Tag::render() const
   ByteVector data;
 
   data.append(fileIdentifier());
-  data.append(TagPrivate::stringHandler->render(d->title).resize(30));
-  data.append(TagPrivate::stringHandler->render(d->artist).resize(30));
-  data.append(TagPrivate::stringHandler->render(d->album).resize(30));
-  data.append(TagPrivate::stringHandler->render(d->year).resize(4));
-  data.append(TagPrivate::stringHandler->render(d->comment).resize(28));
+  data.append(stringHandler->render(d->title).resize(30));
+  data.append(stringHandler->render(d->artist).resize(30));
+  data.append(stringHandler->render(d->album).resize(30));
+  data.append(stringHandler->render(d->year).resize(4));
+  data.append(stringHandler->render(d->comment).resize(28));
   data.append(char(0));
   data.append(char(d->track));
   data.append(char(d->genre));
@@ -142,12 +149,12 @@ String ID3v1::Tag::genre() const
   return ID3v1::genre(d->genre);
 }
 
-TagLib::uint ID3v1::Tag::year() const
+unsigned int ID3v1::Tag::year() const
 {
   return d->year.toInt();
 }
 
-TagLib::uint ID3v1::Tag::track() const
+unsigned int ID3v1::Tag::track() const
 {
   return d->track;
 }
@@ -177,20 +184,32 @@ void ID3v1::Tag::setGenre(const String &s)
   d->genre = ID3v1::genreIndex(s);
 }
 
-void ID3v1::Tag::setYear(uint i)
+void ID3v1::Tag::setYear(unsigned int i)
 {
-  d->year = i > 0 ? String::number(i) : String::null;
+  d->year = i > 0 ? String::number(i) : String();
 }
 
-void ID3v1::Tag::setTrack(uint i)
+void ID3v1::Tag::setTrack(unsigned int i)
 {
   d->track = i < 256 ? i : 0;
 }
 
+unsigned int ID3v1::Tag::genreNumber() const
+{
+  return d->genre;
+}
+
+void ID3v1::Tag::setGenreNumber(unsigned int i)
+{
+  d->genre = i < 256 ? i : 255;
+}
+
 void ID3v1::Tag::setStringHandler(const StringHandler *handler)
 {
-  delete TagPrivate::stringHandler;
-  TagPrivate::stringHandler = handler;
+  if(handler)
+    stringHandler = handler;
+  else
+    stringHandler = &defaultStringHandler;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -202,7 +221,7 @@ void ID3v1::Tag::read()
   if(d->file && d->file->isValid()) {
     d->file->seek(d->tagOffset);
     // read the tag -- always 128 bytes
-    ByteVector data = d->file->readBlock(128);
+    const ByteVector data = d->file->readBlock(128);
 
     // some initial sanity checking
     if(data.size() == 128 && data.startsWith("TAG"))
@@ -216,16 +235,16 @@ void ID3v1::Tag::parse(const ByteVector &data)
 {
   int offset = 3;
 
-  d->title = TagPrivate::stringHandler->parse(data.mid(offset, 30));
+  d->title = stringHandler->parse(data.mid(offset, 30));
   offset += 30;
 
-  d->artist = TagPrivate::stringHandler->parse(data.mid(offset, 30));
+  d->artist = stringHandler->parse(data.mid(offset, 30));
   offset += 30;
 
-  d->album = TagPrivate::stringHandler->parse(data.mid(offset, 30));
+  d->album = stringHandler->parse(data.mid(offset, 30));
   offset += 30;
 
-  d->year = TagPrivate::stringHandler->parse(data.mid(offset, 4));
+  d->year = stringHandler->parse(data.mid(offset, 4));
   offset += 4;
 
   // Check for ID3v1.1 -- Note that ID3v1 *does not* support "track zero" -- this
@@ -236,13 +255,13 @@ void ID3v1::Tag::parse(const ByteVector &data)
   if(data[offset + 28] == 0 && data[offset + 29] != 0) {
     // ID3v1.1 detected
 
-    d->comment = TagPrivate::stringHandler->parse(data.mid(offset, 28));
-    d->track = uchar(data[offset + 29]);
+    d->comment = stringHandler->parse(data.mid(offset, 28));
+    d->track   = static_cast<unsigned char>(data[offset + 29]);
   }
   else
     d->comment = data.mid(offset, 30);
 
   offset += 30;
 
-  d->genre = uchar(data[offset]);
+  d->genre = static_cast<unsigned char>(data[offset]);
 }
