@@ -1,11 +1,11 @@
 /***************************************************************************
-    copyright            : (C) 2002, 2003 by Scott Wheeler
+    copyright            : (C) 2002 - 2008 by Scott Wheeler
     email                : wheeler@kde.org
  ***************************************************************************/
 
 /***************************************************************************
  *   This library is free software; you can redistribute it and/or modify  *
- *   it  under the terms of the GNU Lesser General Public License version  *
+ *   it under the terms of the GNU Lesser General Public License version   *
  *   2.1 as published by the Free Software Foundation.                     *
  *                                                                         *
  *   This library is distributed in the hope that it will be useful, but   *
@@ -15,21 +15,29 @@
  *                                                                         *
  *   You should have received a copy of the GNU Lesser General Public      *
  *   License along with this library; if not, write to the Free Software   *
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
- *   USA                                                                   *
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
+ *   02110-1301  USA                                                       *
+ *                                                                         *
+ *   Alternatively, this file is available under the Mozilla Public        *
+ *   License Version 1.1.  You may obtain a copy of the License at         *
+ *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
 #ifndef TAGLIB_FILE_H
 #define TAGLIB_FILE_H
 
+#include "taglib_export.h"
 #include "taglib.h"
+#include "tag.h"
 #include "tbytevector.h"
+#include "tiostream.h"
 
 namespace TagLib {
 
   class String;
   class Tag;
   class AudioProperties;
+  class PropertyMap;
 
   //! A file class with some useful methods for tag manipulation
 
@@ -39,7 +47,7 @@ namespace TagLib {
    * ByteVector and a binary search method for finding patterns in a file.
    */
 
-  class File
+  class TAGLIB_EXPORT File
   {
   public:
     /*!
@@ -54,6 +62,23 @@ namespace TagLib {
       End
     };
 
+    /*!
+     * Specify which tags to strip either explicitly, or on save.
+     */
+    enum StripTags {
+      StripNone,  //<! Don't strip any tags
+      StripOthers //<! Strip all tags not explicitly referenced in method call
+    };
+
+    /*!
+     * Used to specify if when saving files, if values between different tag
+     * types should be synchronized.
+     */
+    enum DuplicateTags {
+      Duplicate,     //<! Synchronize values between different tag types
+      DoNotDuplicate //<! Do not synchronize values between different tag types
+    };
+
     /*!
      * Destroys this File instance.
      */
@@ -62,7 +87,7 @@ namespace TagLib {
     /*!
      * Returns the file name in the local file system encoding.
      */
-    const char *name() const;
+    FileName name() const;
 
     /*!
      * Returns a pointer to this file's tag.  This should be reimplemented in
@@ -70,6 +95,44 @@ namespace TagLib {
      */
     virtual Tag *tag() const = 0;
 
+    /*!
+     * Exports the tags of the file as dictionary mapping (human readable) tag
+     * names (uppercase Strings) to StringLists of tag values. Calls the according
+     * specialization in the File subclasses.
+     * For each metadata object of the file that could not be parsed into the PropertyMap
+     * format, the returned map's unsupportedData() list will contain one entry identifying
+     * that object (e.g. the frame type for ID3v2 tags). Use removeUnsupportedProperties()
+     * to remove (a subset of) them.
+     * For files that contain more than one tag (e.g. an MP3 with both an ID3v1 and an ID3v2
+     * tag) only the most "modern" one will be exported (ID3v2 in this case).
+     * BIC: Will be made virtual in future releases.
+     */
+    PropertyMap properties() const;
+
+    /*!
+     * Removes unsupported properties, or a subset of them, from the file's metadata.
+     * The parameter \a properties must contain only entries from
+     * properties().unsupportedData().
+     * BIC: Will be mad virtual in future releases.
+     */
+    void removeUnsupportedProperties(const StringList& properties);
+
+    /*!
+     * Sets the tags of this File to those specified in \a properties. Calls the
+     * according specialization method in the subclasses of File to do the translation
+     * into the format-specific details.
+     * If some value(s) could not be written imported to the specific metadata format,
+     * the returned PropertyMap will contain those value(s). Otherwise it will be empty,
+     * indicating that no problems occurred.
+     * With file types that support several tag formats (for instance, MP3 files can have
+     * ID3v1, ID3v2, and APEv2 tags), this function will create the most appropriate one
+     * (ID3v2 for MP3 files). Older formats will be updated as well, if they exist, but won't
+     * be taken into account for the return value of this function.
+     * See the documentation of the subclass implementations for detailed descriptions.
+     * BIC: will become pure virtual in the future
+     */
+    PropertyMap setProperties(const PropertyMap &properties);
+
     /*!
      * Returns a pointer to this file's audio properties.  This should be
      * reimplemented in the concrete subclasses.  If no audio properties were
@@ -80,13 +143,19 @@ namespace TagLib {
     /*!
      * Save the file and its associated tags.  This should be reimplemented in
      * the concrete subclasses.  Returns true if the save succeeds.
+     *
+     * \warning On UNIX multiple processes are able to write to the same file at
+     * the same time.  This can result in serious file corruption.  If you are
+     * developing a program that makes use of TagLib from multiple processes you
+     * must insure that you are only doing writes to a particular file from one
+     * of them.
      */
     virtual bool save() = 0;
 
     /*!
      * Reads a block of size \a length at the current get pointer.
      */
-    ByteVector readBlock(ulong length);
+    ByteVector readBlock(unsigned long length);
 
     /*!
      * Attempts to write the block \a data at the current get pointer.  If the
@@ -103,33 +172,33 @@ namespace TagLib {
      * Returns the offset in the file that \a pattern occurs at or -1 if it can
      * not be found.  If \a before is set, the search will only continue until the
      * pattern \a before is found.  This is useful for tagging purposes to search
-     * for a tag before the synch frame.
+     * for a tag before the sync frame.
      *
      * Searching starts at \a fromOffset, which defaults to the beginning of the
      * file.
      *
-     * \note This has the practial limitation that \a pattern can not be longer
+     * \note This has the practical limitation that \a pattern can not be longer
      * than the buffer size used by readBlock().  Currently this is 1024 bytes.
      */
     long find(const ByteVector &pattern,
               long fromOffset = 0,
-              const ByteVector &before = ByteVector::null);
+              const ByteVector &before = ByteVector());
 
     /*!
      * Returns the offset in the file that \a pattern occurs at or -1 if it can
      * not be found.  If \a before is set, the search will only continue until the
      * pattern \a before is found.  This is useful for tagging purposes to search
-     * for a tag before the synch frame.
+     * for a tag before the sync frame.
      *
      * Searching starts at \a fromOffset and proceeds from the that point to the
      * beginning of the file and defaults to the end of the file.
      *
-     * \note This has the practial limitation that \a pattern can not be longer
+     * \note This has the practical limitation that \a pattern can not be longer
      * than the buffer size used by readBlock().  Currently this is 1024 bytes.
      */
     long rfind(const ByteVector &pattern,
                long fromOffset = 0,
-               const ByteVector &before = ByteVector::null);
+               const ByteVector &before = ByteVector());
 
     /*!
      * Insert \a data at position \a start in the file overwriting \a replace
@@ -138,7 +207,7 @@ namespace TagLib {
      * \note This method is slow since it requires rewriting all of the file
      * after the insertion point.
      */
-    void insert(const ByteVector &data, ulong start = 0, ulong replace = 0);
+    void insert(const ByteVector &data, unsigned long start = 0, unsigned long replace = 0);
 
     /*!
      * Removes a block of the file starting a \a start and continuing for
@@ -147,7 +216,7 @@ namespace TagLib {
      * \note This method is slow since it involves rewriting all of the file
      * after the removed portion.
      */
-    void removeBlock(ulong start = 0, ulong length = 0);
+    void removeBlock(unsigned long start = 0, unsigned long length = 0);
 
     /*!
      * Returns true if the file is read only (or if the file can not be opened).
@@ -161,8 +230,7 @@ namespace TagLib {
     bool isOpen() const;
 
     /*!
-     * Returns true if the file is open and readble and valid information for
-     * the Tag and / or AudioProperties was found.
+     * Returns true if the file is open and readable.
      */
     bool isValid() const;
 
@@ -180,7 +248,7 @@ namespace TagLib {
     void clear();
 
     /*!
-     * Returns the current offset withing the file.
+     * Returns the current offset within the file.
      */
     long tell() const;
 
@@ -192,13 +260,17 @@ namespace TagLib {
     /*!
      * Returns true if \a file can be opened for reading.  If the file does not
      * exist, this will return false.
+     *
+     * \deprecated Use system functions, e.g. access() (_access_s() on Windows).
      */
-    static bool isReadable(const char *file);
+    TAGLIB_DEPRECATED static bool isReadable(const char *file);
 
     /*!
      * Returns true if \a file can be opened for writing.
+     *
+     * \deprecated Use system functions, e.g. access() (_access_s() on Windows).
      */
-    static bool isWritable(const char *name);
+    TAGLIB_DEPRECATED static bool isWritable(const char *name);
 
   protected:
     /*!
@@ -208,7 +280,18 @@ namespace TagLib {
      * \note Constructor is protected since this class should only be
      * instantiated through subclasses.
      */
-    File(const char *file);
+    File(FileName file);
+
+    /*!
+     * Construct a File object and use the \a stream instance.
+     *
+     * \note TagLib will *not* take ownership of the stream, the caller is
+     * responsible for deleting it after the File object.
+     *
+     * \note Constructor is protected since this class should only be
+     * instantiated through subclasses.
+     */
+    File(IOStream *stream);
 
     /*!
      * Marks the file as valid or invalid.
@@ -225,7 +308,7 @@ namespace TagLib {
     /*!
      * Returns the buffer size that is used for internal buffering.
      */
-    static uint bufferSize();
+    static unsigned int bufferSize();
 
   private:
     File(const File &);
