@@ -35,6 +35,9 @@
 using namespace TagLib;
 using namespace XM;
 
+namespace
+{
+
 /*!
  * The Reader classes are helpers to make handling of the stripped XM
  * format more easy. In the stripped XM format certain header sizes might
@@ -130,7 +133,7 @@ public:
   {
     ByteVector data = file.readBlock(std::min(m_size, limit));
     unsigned int count = data.size();
-    int index = data.find((char) 0);
+    int index = data.find(static_cast<char>(0));
     if(index > -1) {
       data.resize(index);
     }
@@ -343,11 +346,13 @@ private:
   List<Reader*> m_readers;
 };
 
+} // namespace
+
 class XM::File::FilePrivate
 {
 public:
   FilePrivate(AudioProperties::ReadStyle propertiesStyle)
-    : tag(), properties(propertiesStyle)
+    :  properties(propertiesStyle)
   {
   }
 
@@ -534,7 +539,7 @@ void XM::File::read(bool)
         .u16L(bpmSpeed);
 
   unsigned int count = header.read(*this, headerSize - 4U);
-  unsigned int size = std::min(headerSize - 4U, (unsigned long)header.size());
+  unsigned int size = std::min(headerSize - 4U, static_cast<unsigned long>(header.size()));
 
   READ_ASSERT(count == size);
 
@@ -566,7 +571,7 @@ void XM::File::read(bool)
     seek(patternHeaderLength - (4 + count) + dataSize, Current);
   }
 
-  StringList intrumentNames;
+  StringList instrumentNames;
   StringList sampleNames;
   unsigned int sumSampleCount = 0;
 
@@ -586,13 +591,13 @@ void XM::File::read(bool)
     unsigned int count = 4 + instrument.read(*this, instrumentHeaderSize - 4U);
     READ_ASSERT(count == std::min(instrumentHeaderSize, (unsigned long)instrument.size() + 4));
 
-    unsigned long sampleHeaderSize = 0;
     long offset = 0;
     if(sampleCount > 0) {
+      unsigned long sampleHeaderSize = 0;
       sumSampleCount += sampleCount;
       // wouldn't know which header size to assume otherwise:
       READ_ASSERT(instrumentHeaderSize >= count + 4 && readU32L(sampleHeaderSize));
-      // skip unhandeled header proportion:
+      // skip unhandled header proportion:
       seek(instrumentHeaderSize - count - 4, Current);
 
       for(unsigned short j = 0; j < sampleCount; ++ j) {
@@ -620,7 +625,7 @@ void XM::File::read(bool)
 
         unsigned int count = sample.read(*this, sampleHeaderSize);
         READ_ASSERT(count == std::min(sampleHeaderSize, (unsigned long)sample.size()));
-        // skip unhandeled header proportion:
+        // skip unhandled header proportion:
         seek(sampleHeaderSize - count, Current);
 
         offset += sampleLength;
@@ -630,12 +635,12 @@ void XM::File::read(bool)
     else {
       offset = instrumentHeaderSize - count;
     }
-    intrumentNames.append(instrumentName);
+    instrumentNames.append(instrumentName);
     seek(offset, Current);
   }
 
   d->properties.setSampleCount(sumSampleCount);
-  String comment(intrumentNames.toString("\n"));
+  String comment(instrumentNames.toString("\n"));
   if(!sampleNames.isEmpty()) {
     comment += "\n";
     comment += sampleNames.toString("\n");
