@@ -1,4 +1,4 @@
-/***************************************************************************
+ /***************************************************************************
     copyright           : (C) 2007 by Lukas Lalinsky
     email               : lukas@oxygene.sk
  ***************************************************************************/
@@ -42,10 +42,14 @@
 #include <unknownframe.h>
 #include <chapterframe.h>
 #include <tableofcontentsframe.h>
+#include <commentsframe.h>
+#include <podcastframe.h>
+#include <privateframe.h>
 #include <tdebug.h>
 #include <tpropertymap.h>
 #include <tzlib.h>
 #include <cppunit/extensions/HelperMacros.h>
+#include "plainfile.h"
 #include "utils.h"
 
 using namespace std;
@@ -56,8 +60,8 @@ class PublicFrame : public ID3v2::Frame
   public:
     PublicFrame() : ID3v2::Frame(ByteVector("XXXX\0\0\0\0\0\0", 10)) {}
     String readStringField(const ByteVector &data, String::Type encoding,
-                           int *positon = 0)
-      { return ID3v2::Frame::readStringField(data, encoding, positon); }
+                           int *position = 0)
+      { return ID3v2::Frame::readStringField(data, encoding, position); }
     virtual String toString() const { return String(); }
     virtual void parseFields(const ByteVector &) {}
     virtual ByteVector renderFields() const { return ByteVector(); }
@@ -75,16 +79,20 @@ class TestID3v2 : public CppUnit::TestFixture
   CPPUNIT_TEST(testParseAPIC);
   CPPUNIT_TEST(testParseAPIC_UTF16_BOM);
   CPPUNIT_TEST(testParseAPICv22);
+  CPPUNIT_TEST(testRenderAPIC);
   CPPUNIT_TEST(testDontRender22);
   CPPUNIT_TEST(testParseGEOB);
+  CPPUNIT_TEST(testRenderGEOB);
   CPPUNIT_TEST(testPOPMtoString);
   CPPUNIT_TEST(testParsePOPM);
   CPPUNIT_TEST(testParsePOPMWithoutCounter);
   CPPUNIT_TEST(testRenderPOPM);
   CPPUNIT_TEST(testPOPMFromFile);
   CPPUNIT_TEST(testParseRelativeVolumeFrame);
+  CPPUNIT_TEST(testRenderRelativeVolumeFrame);
   CPPUNIT_TEST(testParseUniqueFileIdentifierFrame);
   CPPUNIT_TEST(testParseEmptyUniqueFileIdentifierFrame);
+  CPPUNIT_TEST(testRenderUniqueFileIdentifierFrame);
   CPPUNIT_TEST(testBrokenFrame1);
   CPPUNIT_TEST(testItunes24FrameSize);
   CPPUNIT_TEST(testParseUrlLinkFrame);
@@ -98,9 +106,16 @@ class TestID3v2 : public CppUnit::TestFixture
   CPPUNIT_TEST(testRenderSynchronizedLyricsFrame);
   CPPUNIT_TEST(testParseEventTimingCodesFrame);
   CPPUNIT_TEST(testRenderEventTimingCodesFrame);
+  CPPUNIT_TEST(testParseCommentsFrame);
+  CPPUNIT_TEST(testRenderCommentsFrame);
+  CPPUNIT_TEST(testParsePodcastFrame);
+  CPPUNIT_TEST(testRenderPodcastFrame);
+  CPPUNIT_TEST(testParsePrivateFrame);
+  CPPUNIT_TEST(testRenderPrivateFrame);
   CPPUNIT_TEST(testSaveUTF16Comment);
   CPPUNIT_TEST(testUpdateGenre23_1);
   CPPUNIT_TEST(testUpdateGenre23_2);
+  CPPUNIT_TEST(testUpdateGenre23_3);
   CPPUNIT_TEST(testUpdateGenre24);
   CPPUNIT_TEST(testUpdateDate22);
   CPPUNIT_TEST(testDowngradeTo23);
@@ -109,6 +124,8 @@ class TestID3v2 : public CppUnit::TestFixture
   CPPUNIT_TEST(testW000);
   CPPUNIT_TEST(testPropertyInterface);
   CPPUNIT_TEST(testPropertyInterface2);
+  CPPUNIT_TEST(testPropertiesMovement);
+  CPPUNIT_TEST(testPropertyGrouping);
   CPPUNIT_TEST(testDeleteFrame);
   CPPUNIT_TEST(testSaveAndStripID3v1ShouldNotAddFrameFromID3v1ToId3v2);
   CPPUNIT_TEST(testParseChapterFrame);
@@ -118,6 +135,7 @@ class TestID3v2 : public CppUnit::TestFixture
   CPPUNIT_TEST(testShrinkPadding);
   CPPUNIT_TEST(testEmptyFrame);
   CPPUNIT_TEST(testDuplicateTags);
+  CPPUNIT_TEST(testParseTOCFrameWithManyChildren);
   CPPUNIT_TEST_SUITE_END();
 
 public:
@@ -142,7 +160,7 @@ public:
 
     MPEG::File file(newname.c_str());
     file.ID3v2Tag(true)->addFrame(f);
-    file.save(MPEG::File::ID3v2, true, 3);
+    file.save(MPEG::File::ID3v2, File::StripOthers, ID3v2::v3);
     CPPUNIT_ASSERT_EQUAL(true, file.hasID3v2Tag());
 
     ByteVector data = f->render();
@@ -163,7 +181,7 @@ public:
 
     MPEG::File file(copy.fileName().c_str());
     file.ID3v2Tag(true)->addFrame(f);
-    file.save(MPEG::File::ID3v2, true, 3);
+    file.save(MPEG::File::ID3v2, File::StripOthers, ID3v2::v3);
     CPPUNIT_ASSERT(file.hasID3v2Tag());
 
     ByteVector data = f->render();
@@ -181,7 +199,14 @@ public:
     sl.append("Foo");
     sl.append("Bar");
     f.setText(sl);
-    CPPUNIT_ASSERT_EQUAL((unsigned int)(4+4+2+1+6+2+6), f.render().size());
+    ByteVector data = f.render();
+    CPPUNIT_ASSERT_EQUAL((unsigned int)(4+4+2+1+6+2+6), data.size());
+    ByteVector noBomBeData("TPE1\x00\x00\x00\x0f\x00\x00\x02"
+                           "\0F\0o\0o\0\0"
+                           "\0B\0a\0r", 25);
+    CPPUNIT_ASSERT_EQUAL(noBomBeData, data);
+    f.setData(data);
+    CPPUNIT_ASSERT_EQUAL(String("Foo Bar"), f.toString());
   }
 
   void testUTF16Delimiter()
@@ -191,7 +216,32 @@ public:
     sl.append("Foo");
     sl.append("Bar");
     f.setText(sl);
-    CPPUNIT_ASSERT_EQUAL((unsigned int)(4+4+2+1+8+2+8), f.render().size());
+    ByteVector data = f.render();
+    CPPUNIT_ASSERT_EQUAL((unsigned int)(4+4+2+1+8+2+8), data.size());
+    ByteVector multiBomLeData("TPE1\x00\x00\x00\x13\x00\x00\x01\xff\xfe"
+                              "F\0o\0o\0\0\0" "\xff\xfe"
+                              "B\0a\0r\0", 29);
+    CPPUNIT_ASSERT_EQUAL(multiBomLeData, data);
+    f.setData(data);
+    CPPUNIT_ASSERT_EQUAL(String("Foo Bar"), f.toString());
+
+    ByteVector multiBomBeData("TPE1\x00\x00\x00\x13\x00\x00\x01\xfe\xff"
+                              "\0F\0o\0o\0\0" "\xfe\xff"
+                              "\0B\0a\0r", 29);
+    f.setData(multiBomBeData);
+    CPPUNIT_ASSERT_EQUAL(String("Foo Bar"), f.toString());
+
+    ByteVector singleBomLeData("TPE1\x00\x00\x00\x13\x00\x00\x01\xff\xfe"
+                               "F\0o\0o\0\0\0"
+                               "B\0a\0r\0", 27);
+    f.setData(singleBomLeData);
+    CPPUNIT_ASSERT_EQUAL(String("Foo Bar"), f.toString());
+
+    ByteVector singleBomBeData("TPE1\x00\x00\x00\x13\x00\x00\x01\xfe\xff"
+                               "\0F\0o\0o\0\0"
+                               "\0B\0a\0r", 27);
+    f.setData(singleBomBeData);
+    CPPUNIT_ASSERT_EQUAL(String("Foo Bar"), f.toString());
   }
 
   void testBrokenFrame1()
@@ -248,8 +298,10 @@ public:
                                  "\x01"
                                  "d\x00"
                                  "\x00", 14);
+    ID3v2::Header header;
+    header.setMajorVersion(2);
     ID3v2::AttachedPictureFrame *frame =
-        dynamic_cast<TagLib::ID3v2::AttachedPictureFrame*>(factory->createFrame(data, (unsigned int)2));
+      dynamic_cast<TagLib::ID3v2::AttachedPictureFrame *>(factory->createFrame(data, &header));
 
     CPPUNIT_ASSERT(frame);
     CPPUNIT_ASSERT_EQUAL(String("image/jpeg"), frame->mimeType());
@@ -259,6 +311,26 @@ public:
     delete frame;
   }
 
+  void testRenderAPIC()
+  {
+    ID3v2::AttachedPictureFrame f;
+    f.setTextEncoding(String::UTF8);
+    f.setMimeType("image/png");
+    f.setType(ID3v2::AttachedPictureFrame::BackCover);
+    f.setDescription("Description");
+    f.setPicture("PNG data");
+    CPPUNIT_ASSERT_EQUAL(
+      ByteVector("APIC"
+                 "\x00\x00\x00\x20"
+                 "\x00\x00"
+                 "\x03"
+                 "image/png\x00"
+                 "\x04"
+                 "Description\x00"
+                 "PNG data", 42),
+      f.render());
+  }
+
   void testDontRender22()
   {
     ID3v2::FrameFactory *factory = ID3v2::FrameFactory::instance();
@@ -269,8 +341,10 @@ public:
                                  "\x01"
                                  "d\x00"
                                  "\x00", 14);
+    ID3v2::Header header;
+    header.setMajorVersion(2);
     ID3v2::UnknownFrame *frame =
-        dynamic_cast<TagLib::ID3v2::UnknownFrame*>(factory->createFrame(data, (unsigned int)2));
+      dynamic_cast<TagLib::ID3v2::UnknownFrame*>(factory->createFrame(data, &header));
 
     CPPUNIT_ASSERT(frame);
 
@@ -295,6 +369,26 @@ public:
     CPPUNIT_ASSERT_EQUAL(String("d"), f.description());
   }
 
+  void testRenderGEOB()
+  {
+    ID3v2::GeneralEncapsulatedObjectFrame f;
+    f.setTextEncoding(String::Latin1);
+    f.setMimeType("application/octet-stream");
+    f.setFileName("test.bin");
+    f.setDescription("Description");
+    f.setObject(ByteVector(3, '\x01'));
+    CPPUNIT_ASSERT_EQUAL(
+      ByteVector("GEOB"
+                 "\x00\x00\x00\x32"
+                 "\x00\x00"
+                 "\x00"
+                 "application/octet-stream\x00"
+                 "test.bin\x00"
+                 "Description\x00"
+                 "\x01\x01\x01", 60),
+      f.render());
+  }
+
   void testParsePOPM()
   {
     ID3v2::PopularimeterFrame f(ByteVector("POPM"
@@ -383,10 +477,36 @@ public:
     CPPUNIT_ASSERT_EQUAL(String("ident"), f.identification());
     CPPUNIT_ASSERT_EQUAL(15.0f / 512.0f,
                          f.volumeAdjustment(ID3v2::RelativeVolumeFrame::FrontRight));
+    CPPUNIT_ASSERT_EQUAL(static_cast<short>(15),
+                         f.volumeAdjustmentIndex(ID3v2::RelativeVolumeFrame::FrontRight));
     CPPUNIT_ASSERT_EQUAL((unsigned char)8,
                          f.peakVolume(ID3v2::RelativeVolumeFrame::FrontRight).bitsRepresentingPeak);
     CPPUNIT_ASSERT_EQUAL(ByteVector("\x45"),
                          f.peakVolume(ID3v2::RelativeVolumeFrame::FrontRight).peakVolume);
+    const List<ID3v2::RelativeVolumeFrame::ChannelType> channels = f.channels();
+    CPPUNIT_ASSERT_EQUAL(1U, channels.size());
+    CPPUNIT_ASSERT_EQUAL(ID3v2::RelativeVolumeFrame::FrontRight, channels[0]);
+  }
+
+  void testRenderRelativeVolumeFrame()
+  {
+    ID3v2::RelativeVolumeFrame f;
+    f.setIdentification("ident");
+    f.setVolumeAdjustment(15.0f / 512.0f, ID3v2::RelativeVolumeFrame::FrontRight);
+    ID3v2::RelativeVolumeFrame::PeakVolume peakVolume;
+    peakVolume.bitsRepresentingPeak = 8;
+    peakVolume.peakVolume.setData("\x45");
+    f.setPeakVolume(peakVolume, ID3v2::RelativeVolumeFrame::FrontRight);
+    CPPUNIT_ASSERT_EQUAL(
+      ByteVector("RVA2"
+                 "\x00\x00\x00\x0B"
+                 "\x00\x00"
+                 "ident\x00"
+                 "\x02"
+                 "\x00\x0F"
+                 "\x08"
+                 "\x45", 21),
+      f.render());
   }
 
   void testParseUniqueFileIdentifierFrame()
@@ -417,6 +537,18 @@ public:
                          f.identifier());
   }
 
+  void testRenderUniqueFileIdentifierFrame()
+  {
+    ID3v2::UniqueFileIdentifierFrame f("owner", "\x01\x02\x03");
+    CPPUNIT_ASSERT_EQUAL(
+      ByteVector("UFID"
+                 "\x00\x00\x00\x09"
+                 "\x00\x00"
+                 "owner\x00"
+                 "\x01\x02\x03", 19),
+      f.render());
+  }
+
   void testParseUrlLinkFrame()
   {
     ID3v2::UrlLinkFrame f(
@@ -626,6 +758,89 @@ public:
       f.render());
   }
 
+  void testParseCommentsFrame()
+  {
+    ID3v2::CommentsFrame f(
+      ByteVector("COMM"
+                 "\x00\x00\x00\x14"
+                 "\x00\x00"
+                 "\x03"
+                 "deu"
+                 "Description\x00"
+                 "Text", 30));
+    CPPUNIT_ASSERT_EQUAL(String::UTF8, f.textEncoding());
+    CPPUNIT_ASSERT_EQUAL(ByteVector("deu"), f.language());
+    CPPUNIT_ASSERT_EQUAL(String("Description"), f.description());
+    CPPUNIT_ASSERT_EQUAL(String("Text"), f.text());
+  }
+
+  void testRenderCommentsFrame()
+  {
+    ID3v2::CommentsFrame f;
+    f.setTextEncoding(String::UTF16);
+    f.setLanguage("eng");
+    f.setDescription("Description");
+    f.setText("Text");
+    CPPUNIT_ASSERT_EQUAL(
+      ByteVector("COMM"
+                 "\x00\x00\x00\x28"
+                 "\x00\x00"
+                 "\x01"
+                 "eng"
+                 "\xff\xfe" "D\0e\0s\0c\0r\0i\0p\0t\0i\0o\0n\0" "\x00\x00"
+                 "\xff\xfe" "T\0e\0x\0t\0", 50),
+      f.render());
+  }
+
+  void testParsePodcastFrame()
+  {
+    ID3v2::FrameFactory *factory = ID3v2::FrameFactory::instance();
+    ByteVector data = ByteVector("PCST"
+                                 "\x00\x00\x00\x04"
+                                 "\x00\x00"
+                                 "\x00\x00\x00\x00", 14);
+    const ID3v2::Header header;
+    CPPUNIT_ASSERT(dynamic_cast<ID3v2::PodcastFrame *>(
+                     factory->createFrame(data, &header)));
+  }
+
+  void testRenderPodcastFrame()
+  {
+    ID3v2::PodcastFrame f;
+    CPPUNIT_ASSERT_EQUAL(
+      ByteVector("PCST"
+                 "\x00\x00\x00\x04"
+                 "\x00\x00"
+                 "\x00\x00\x00\x00", 14),
+      f.render());
+  }
+
+  void testParsePrivateFrame()
+  {
+    ID3v2::PrivateFrame f(
+      ByteVector("PRIV"
+                 "\x00\x00\x00\x0e"
+                 "\x00\x00"
+                 "WM/Provider\x00"
+                 "TL", 24));
+    CPPUNIT_ASSERT_EQUAL(String("WM/Provider"), f.owner());
+    CPPUNIT_ASSERT_EQUAL(ByteVector("TL"), f.data());
+  }
+
+  void testRenderPrivateFrame()
+  {
+    ID3v2::PrivateFrame f;
+    f.setOwner("WM/Provider");
+    f.setData("TL");
+    CPPUNIT_ASSERT_EQUAL(
+      ByteVector("PRIV"
+                 "\x00\x00\x00\x0e"
+                 "\x00\x00"
+                 "WM/Provider\x00"
+                 "TL", 24),
+      f.render());
+  }
+
   void testItunes24FrameSize()
   {
     MPEG::File f(TEST_FILE_PATH_C("005411.id3"), false);
@@ -662,8 +877,10 @@ public:
                                  "\x00\x00"             // Frame flags
                                  "\x00"                 // Encoding
                                  "(22)Death Metal", 26);     // Text
+    ID3v2::Header header;
+    header.setMajorVersion(3);
     ID3v2::TextIdentificationFrame *frame =
-        dynamic_cast<TagLib::ID3v2::TextIdentificationFrame*>(factory->createFrame(data, (unsigned int)3));
+      dynamic_cast<TagLib::ID3v2::TextIdentificationFrame*>(factory->createFrame(data, &header));
     CPPUNIT_ASSERT_EQUAL((unsigned int)1, frame->fieldList().size());
     CPPUNIT_ASSERT_EQUAL(String("Death Metal"), frame->fieldList()[0]);
 
@@ -677,12 +894,14 @@ public:
     // "Refinement" is different from the ID3v1 genre
     ID3v2::FrameFactory *factory = ID3v2::FrameFactory::instance();
     ByteVector data = ByteVector("TCON"                 // Frame ID
-                                 "\x00\x00\x00\x13"     // Frame size
+                                 "\x00\x00\x00\x0d"     // Frame size
                                  "\x00\x00"             // Frame flags
                                  "\x00"                 // Encoding
                                  "(4)Eurodisco", 23);   // Text
+    ID3v2::Header header;
+    header.setMajorVersion(3);
     ID3v2::TextIdentificationFrame *frame =
-        dynamic_cast<TagLib::ID3v2::TextIdentificationFrame*>(factory->createFrame(data, (unsigned int)3));
+      dynamic_cast<TagLib::ID3v2::TextIdentificationFrame*>(factory->createFrame(data, &header));
     CPPUNIT_ASSERT_EQUAL((unsigned int)2, frame->fieldList().size());
     CPPUNIT_ASSERT_EQUAL(String("4"), frame->fieldList()[0]);
     CPPUNIT_ASSERT_EQUAL(String("Eurodisco"), frame->fieldList()[1]);
@@ -692,6 +911,29 @@ public:
     CPPUNIT_ASSERT_EQUAL(String("Disco Eurodisco"), tag.genre());
   }
 
+  void testUpdateGenre23_3()
+  {
+    // Multiple references and a refinement
+    ID3v2::FrameFactory *factory = ID3v2::FrameFactory::instance();
+    ByteVector data = ByteVector("TCON"                 // Frame ID
+                                 "\x00\x00\x00\x15"     // Frame size
+                                 "\x00\x00"             // Frame flags
+                                 "\x00"                 // Encoding
+                                 "(9)(138)Viking Metal", 31);   // Text
+    ID3v2::Header header;
+    header.setMajorVersion(3);
+    ID3v2::TextIdentificationFrame *frame =
+      dynamic_cast<TagLib::ID3v2::TextIdentificationFrame*>(factory->createFrame(data, &header));
+    CPPUNIT_ASSERT_EQUAL(3U, frame->fieldList().size());
+    CPPUNIT_ASSERT_EQUAL(String("9"), frame->fieldList()[0]);
+    CPPUNIT_ASSERT_EQUAL(String("138"), frame->fieldList()[1]);
+    CPPUNIT_ASSERT_EQUAL(String("Viking Metal"), frame->fieldList()[2]);
+
+    ID3v2::Tag tag;
+    tag.addFrame(frame);
+    CPPUNIT_ASSERT_EQUAL(String("Metal Black Metal Viking Metal"), tag.genre());
+  }
+
   void testUpdateGenre24()
   {
     ID3v2::FrameFactory *factory = ID3v2::FrameFactory::instance();
@@ -700,8 +942,9 @@ public:
                                  "\x00\x00"               // Frame flags
                                  "\0"                   // Encoding
                                  "14\0Eurodisco", 23);     // Text
+    ID3v2::Header header;
     ID3v2::TextIdentificationFrame *frame =
-        dynamic_cast<TagLib::ID3v2::TextIdentificationFrame*>(factory->createFrame(data, (unsigned int)4));
+      dynamic_cast<TagLib::ID3v2::TextIdentificationFrame*>(factory->createFrame(data, &header));
     CPPUNIT_ASSERT_EQUAL((unsigned int)2, frame->fieldList().size());
     CPPUNIT_ASSERT_EQUAL(String("14"), frame->fieldList()[0]);
     CPPUNIT_ASSERT_EQUAL(String("Eurodisco"), frame->fieldList()[1]);
@@ -745,6 +988,9 @@ public:
       tf = new ID3v2::TextIdentificationFrame("TIPL", String::Latin1);
       tf->setText(StringList().append("Producer").append("Artist 3").append("Mastering").append("Artist 4"));
       foo.ID3v2Tag()->addFrame(tf);
+      tf = new ID3v2::TextIdentificationFrame("TCON", String::Latin1);
+      tf->setText(StringList().append("51").append("Noise").append("Power Noise"));
+      foo.ID3v2Tag()->addFrame(tf);
       foo.ID3v2Tag()->addFrame(new ID3v2::TextIdentificationFrame("TDRL", String::Latin1));
       foo.ID3v2Tag()->addFrame(new ID3v2::TextIdentificationFrame("TDTG", String::Latin1));
       foo.ID3v2Tag()->addFrame(new ID3v2::TextIdentificationFrame("TMOO", String::Latin1));
@@ -753,7 +999,7 @@ public:
       foo.ID3v2Tag()->addFrame(new ID3v2::TextIdentificationFrame("TSOT", String::Latin1));
       foo.ID3v2Tag()->addFrame(new ID3v2::TextIdentificationFrame("TSST", String::Latin1));
       foo.ID3v2Tag()->addFrame(new ID3v2::TextIdentificationFrame("TSOP", String::Latin1));
-      foo.save(MPEG::File::AllTags, true, 3);
+      foo.save(MPEG::File::AllTags, File::StripOthers, ID3v2::v3);
     }
     {
       MPEG::File bar(newname.c_str());
@@ -776,6 +1022,12 @@ public:
       CPPUNIT_ASSERT_EQUAL(String("Artist 3"), tf->fieldList()[5]);
       CPPUNIT_ASSERT_EQUAL(String("Mastering"), tf->fieldList()[6]);
       CPPUNIT_ASSERT_EQUAL(String("Artist 4"), tf->fieldList()[7]);
+      tf = dynamic_cast<ID3v2::TextIdentificationFrame *>(bar.ID3v2Tag()->frameList("TCON").front());
+      CPPUNIT_ASSERT(tf);
+      CPPUNIT_ASSERT_EQUAL(3U, tf->fieldList().size());
+      CPPUNIT_ASSERT_EQUAL(String("51"), tf->fieldList()[0]);
+      CPPUNIT_ASSERT_EQUAL(String("39"), tf->fieldList()[1]);
+      CPPUNIT_ASSERT_EQUAL(String("Power Noise"), tf->fieldList()[2]);
       CPPUNIT_ASSERT(!bar.ID3v2Tag()->frameListMap().contains("TDRL"));
       CPPUNIT_ASSERT(!bar.ID3v2Tag()->frameListMap().contains("TDTG"));
       CPPUNIT_ASSERT(!bar.ID3v2Tag()->frameListMap().contains("TMOO"));
@@ -786,7 +1038,34 @@ public:
       CPPUNIT_ASSERT(!bar.ID3v2Tag()->frameListMap().contains("TSOP"));
 #endif
       CPPUNIT_ASSERT(!bar.ID3v2Tag()->frameListMap().contains("TSST"));
-}
+    }
+    {
+      const ByteVector expectedId3v23Data(
+            "ID3" "\x03\x00\x00\x00\x00\x09\x49"
+            "TSOA" "\x00\x00\x00\x01\x00\x00\x00"
+            "TSOT" "\x00\x00\x00\x01\x00\x00\x00"
+            "TSOP" "\x00\x00\x00\x01\x00\x00\x00"
+            "TORY" "\x00\x00\x00\x05\x00\x00\x00" "2011"
+            "TYER" "\x00\x00\x00\x05\x00\x00\x00" "2012"
+            "TDAT" "\x00\x00\x00\x05\x00\x00\x00" "1704"
+            "TIME" "\x00\x00\x00\x05\x00\x00\x00" "1201"
+            "IPLS" "\x00\x00\x00\x44\x00\x00\x00" "Guitar" "\x00"
+            "Artist 1" "\x00" "Drums" "\x00" "Artist 2" "\x00" "Producer" "\x00"
+            "Artist 3" "\x00" "Mastering" "\x00" "Artist 4"
+            "TCON" "\x00\x00\x00\x14\x00\x00\x00" "(51)(39)Power Noise", 211);
+      const ByteVector actualId3v23Data =
+          PlainFile(newname.c_str()).readBlock(expectedId3v23Data.size());
+      CPPUNIT_ASSERT_EQUAL(expectedId3v23Data, actualId3v23Data);
+    }
+
+    ScopedFileCopy rareFramesCopy("rare_frames", ".mp3");
+
+    {
+      MPEG::File f(rareFramesCopy.fileName().c_str());
+      f.save(MPEG::File::AllTags, File::StripOthers, ID3v2::v3);
+      f.seek(f.find("TCON") + 11);
+      CPPUNIT_ASSERT_EQUAL(ByteVector("(13)"), f.readBlock(4));
+    }
   }
 
   void testCompressedFrameWithBrokenLength()
@@ -920,6 +1199,82 @@ public:
     CPPUNIT_ASSERT_EQUAL(frame6, ID3v2::UniqueFileIdentifierFrame::findByOwner(&tag, "http://musicbrainz.org"));
   }
 
+  void testPropertiesMovement()
+  {
+    ID3v2::Tag tag;
+    ID3v2::TextIdentificationFrame *frameMvnm = new ID3v2::TextIdentificationFrame("MVNM");
+    frameMvnm->setText("Movement Name");
+    tag.addFrame(frameMvnm);
+
+    ID3v2::TextIdentificationFrame *frameMvin = new ID3v2::TextIdentificationFrame("MVIN");
+    frameMvin->setText("2/3");
+    tag.addFrame(frameMvin);
+
+    PropertyMap properties = tag.properties();
+    CPPUNIT_ASSERT(properties.contains("MOVEMENTNAME"));
+    CPPUNIT_ASSERT(properties.contains("MOVEMENTNUMBER"));
+    CPPUNIT_ASSERT_EQUAL(String("Movement Name"), properties["MOVEMENTNAME"].front());
+    CPPUNIT_ASSERT_EQUAL(String("2/3"), properties["MOVEMENTNUMBER"].front());
+
+    ByteVector frameDataMvnm("MVNM"
+                             "\x00\x00\x00\x0e"
+                             "\x00\x00"
+                             "\x00"
+                             "Movement Name", 24);
+    CPPUNIT_ASSERT_EQUAL(frameDataMvnm, frameMvnm->render());
+    ByteVector frameDataMvin("MVIN"
+                             "\x00\x00\x00\x04"
+                             "\x00\x00"
+                             "\x00"
+                             "2/3", 14);
+    CPPUNIT_ASSERT_EQUAL(frameDataMvin, frameMvin->render());
+
+    ID3v2::FrameFactory *factory = ID3v2::FrameFactory::instance();
+    ID3v2::Header header;
+    ID3v2::TextIdentificationFrame *parsedFrameMvnm =
+      dynamic_cast<ID3v2::TextIdentificationFrame *>(
+        factory->createFrame(frameDataMvnm, &header));
+    ID3v2::TextIdentificationFrame *parsedFrameMvin =
+      dynamic_cast<ID3v2::TextIdentificationFrame *>(
+        factory->createFrame(frameDataMvin, &header));
+    CPPUNIT_ASSERT(parsedFrameMvnm);
+    CPPUNIT_ASSERT(parsedFrameMvin);
+    CPPUNIT_ASSERT_EQUAL(String("Movement Name"), parsedFrameMvnm->toString());
+    CPPUNIT_ASSERT_EQUAL(String("2/3"), parsedFrameMvin->toString());
+
+    tag.addFrame(parsedFrameMvnm);
+    tag.addFrame(parsedFrameMvin);
+  }
+
+  void testPropertyGrouping()
+  {
+    ID3v2::Tag tag;
+    ID3v2::TextIdentificationFrame *frameGrp1 = new ID3v2::TextIdentificationFrame("GRP1");
+    frameGrp1->setText("Grouping");
+    tag.addFrame(frameGrp1);
+
+    PropertyMap properties = tag.properties();
+    CPPUNIT_ASSERT(properties.contains("GROUPING"));
+    CPPUNIT_ASSERT_EQUAL(String("Grouping"), properties["GROUPING"].front());
+
+    ByteVector frameDataGrp1("GRP1"
+                             "\x00\x00\x00\x09"
+                             "\x00\x00"
+                             "\x00"
+                             "Grouping", 19);
+    CPPUNIT_ASSERT_EQUAL(frameDataGrp1, frameGrp1->render());
+
+    ID3v2::FrameFactory *factory = ID3v2::FrameFactory::instance();
+    ID3v2::Header header;
+    ID3v2::TextIdentificationFrame *parsedFrameGrp1 =
+      dynamic_cast<ID3v2::TextIdentificationFrame *>(
+        factory->createFrame(frameDataGrp1, &header));
+    CPPUNIT_ASSERT(parsedFrameGrp1);
+    CPPUNIT_ASSERT_EQUAL(String("Grouping"), parsedFrameGrp1->toString());
+
+    tag.addFrame(parsedFrameGrp1);
+  }
+
   void testDeleteFrame()
   {
     ScopedFileCopy copy("rare_frames", ".mp3");
@@ -955,7 +1310,7 @@ public:
       MPEG::File bar(newname.c_str());
       bar.ID3v2Tag()->removeFrames("TPE1");
       // Should strip ID3v1 here and not add old values to ID3v2 again
-      bar.save(MPEG::File::ID3v2, true);
+      bar.save(MPEG::File::ID3v2, File::StripOthers);
     }
 
     MPEG::File f(newname.c_str());
@@ -1097,6 +1452,16 @@ public:
     CPPUNIT_ASSERT((unsigned int)0x01 == f.embeddedFrameList().size());
     CPPUNIT_ASSERT(f.embeddedFrameList("TIT2").size() == 1);
     CPPUNIT_ASSERT(f.embeddedFrameList("TIT2")[0]->toString() == "TC1");
+
+    f.removeChildElement("E"); // not existing
+    CPPUNIT_ASSERT_EQUAL(2U, f.entryCount());
+    f.removeChildElement("C");
+    CPPUNIT_ASSERT_EQUAL(1U, f.entryCount());
+    CPPUNIT_ASSERT_EQUAL(ByteVector("D"), f.childElements()[0]);
+
+    ID3v2::Frame *frame = f.embeddedFrameList("TIT2")[0];
+    f.removeEmbeddedFrame(frame);
+    CPPUNIT_ASSERT(f.embeddedFrameList("TIT2").isEmpty());
   }
 
   void testRenderTableOfContentsFrame()
@@ -1135,15 +1500,15 @@ public:
 
     {
       MPEG::File f(newname.c_str());
-      f.ID3v2Tag()->setTitle(std::wstring(64 * 1024, L'X').c_str());
-      f.save(MPEG::File::ID3v2, true);
+      f.ID3v2Tag()->setTitle(longText(64 * 1024));
+      f.save(MPEG::File::ID3v2, File::StripOthers);
     }
     {
       MPEG::File f(newname.c_str());
       CPPUNIT_ASSERT(f.hasID3v2Tag());
       CPPUNIT_ASSERT_EQUAL(74789L, f.length());
       f.ID3v2Tag()->setTitle("ABCDEFGHIJ");
-      f.save(MPEG::File::ID3v2, true);
+      f.save(MPEG::File::ID3v2, File::StripOthers);
     }
     {
       MPEG::File f(newname.c_str());
@@ -1201,7 +1566,7 @@ public:
       CPPUNIT_ASSERT_EQUAL(44100, f.audioProperties()->sampleRate());
 
       f.ID3v2Tag()->setArtist("Artist A");
-      f.save(MPEG::File::ID3v2, true);
+      f.save(MPEG::File::ID3v2, File::StripOthers);
     }
     {
       MPEG::File f(copy.fileName().c_str());
@@ -1217,6 +1582,63 @@ public:
     }
   }
 
+  void testParseTOCFrameWithManyChildren()
+  {
+    MPEG::File f(TEST_FILE_PATH_C("toc_many_children.mp3"));
+    CPPUNIT_ASSERT(f.isValid());
+
+    ID3v2::Tag *tag = f.ID3v2Tag();
+    const ID3v2::FrameList &frames = tag->frameList();
+    CPPUNIT_ASSERT_EQUAL(130U, frames.size());
+    int i = 0;
+    for(ID3v2::FrameList::ConstIterator it = frames.begin(); it != frames.end();
+        ++it, ++i) {
+      if(i > 0) {
+        CPPUNIT_ASSERT_EQUAL(ByteVector("CHAP"), (*it)->frameID());
+        const ID3v2::ChapterFrame *chapFrame =
+            dynamic_cast<const ID3v2::ChapterFrame *>(*it);
+        CPPUNIT_ASSERT_EQUAL(ByteVector("chapter") +
+                             ByteVector(String::number(i - 1).toCString()),
+                             chapFrame->elementID());
+        CPPUNIT_ASSERT_EQUAL(static_cast<unsigned int>(100 * i),
+                             chapFrame->startTime());
+        CPPUNIT_ASSERT_EQUAL(static_cast<unsigned int>(100 * i),
+                             chapFrame->endTime());
+        const ID3v2::FrameList &embeddedFrames = chapFrame->embeddedFrameList();
+        CPPUNIT_ASSERT_EQUAL(1U, embeddedFrames.size());
+        const ID3v2::TextIdentificationFrame *tit2Frame =
+            dynamic_cast<const ID3v2::TextIdentificationFrame *>(
+              embeddedFrames.front());
+        CPPUNIT_ASSERT(tit2Frame);
+        CPPUNIT_ASSERT_EQUAL(String("Marker ") + String::number(i),
+                             tit2Frame->fieldList().front());
+      }
+      else {
+        CPPUNIT_ASSERT_EQUAL(ByteVector("CTOC"), (*it)->frameID());
+        const ID3v2::TableOfContentsFrame *ctocFrame =
+            dynamic_cast<const ID3v2::TableOfContentsFrame *>(*it);
+        CPPUNIT_ASSERT_EQUAL(ByteVector("toc"), ctocFrame->elementID());
+        CPPUNIT_ASSERT(!ctocFrame->isTopLevel());
+        CPPUNIT_ASSERT(!ctocFrame->isOrdered());
+        CPPUNIT_ASSERT_EQUAL(129U, ctocFrame->entryCount());
+        const ID3v2::FrameList &embeddedFrames = ctocFrame->embeddedFrameList();
+        CPPUNIT_ASSERT_EQUAL(1U, embeddedFrames.size());
+        const ID3v2::TextIdentificationFrame *tit2Frame =
+            dynamic_cast<const ID3v2::TextIdentificationFrame *>(
+              embeddedFrames.front());
+        CPPUNIT_ASSERT(tit2Frame);
+        CPPUNIT_ASSERT_EQUAL(StringList("toplevel toc"), tit2Frame->fieldList());
+      }
+    }
+
+    CPPUNIT_ASSERT(!ID3v2::ChapterFrame::findByElementID(tag, "chap2"));
+    CPPUNIT_ASSERT(ID3v2::ChapterFrame::findByElementID(tag, "chapter2"));
+
+    CPPUNIT_ASSERT(!ID3v2::TableOfContentsFrame::findTopLevel(tag));
+    CPPUNIT_ASSERT(!ID3v2::TableOfContentsFrame::findByElementID(tag, "ctoc"));
+    CPPUNIT_ASSERT(ID3v2::TableOfContentsFrame::findByElementID(tag, "toc"));
+  }
+
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION(TestID3v2);
